/**
 * @file        HTTP请求日志插件
 * @author      hotok
 * @date        2024-01-15
 * @lastEditor  hotok
 * @lastEditTime 2024-01-15
 * @description 记录HTTP请求和响应的详细信息，包括请求时间、响应时间、客户端信息等
 */

import { Elysia } from 'elysia';
import Logger from '@/utils/logger';
import { browserInfo } from '@/utils/deviceInfo';
import { formatFileSize } from '@/utils/formatFileSize';
import { getRandomBackgroundColor } from '@/utils/randomChalk';

/**
 * HTTP请求日志插件
 * @description 为Elysia应用添加请求和响应日志功能
 * @param app Elysia应用实例
 * @returns 配置了日志功能的Elysia应用
 * @example
 * const app = new Elysia().use(loggerPlugin)
 */
const loggerPlugin = (app: Elysia) =>
    app
        /** 注册日志实例到应用状态 */
        .decorate('log', Logger)
        /** 注册请求开始时间到应用状态，用于计算响应时间 */
        .state('requestStart', null as [number, number] | null)
        .state('color', null as string | null)
        /** 请求拦截器 - 记录请求信息 */
        .onRequest(({ store: { requestStart, color }, request, server, path, log }) => {
            /** 记录请求开始时间 */
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            requestStart = process.hrtime();
            /** 获取客户端IP信息 */
            const clientIP = server?.requestIP(request);
            color = getRandomBackgroundColor()(' ');
            /** 记录请求日志 */
            log.http({
                type: 'request',
                method: request.method,
                path,
                color: color,
                ip: `${clientIP?.family}: ${clientIP?.address}:${clientIP?.port}`,
                browser: browserInfo(request.headers.get('user-agent')),
            });
        })
        /** 响应拦截器 - 记录响应信息 */
        .onAfterResponse(({ log, store: { requestStart, color }, request, set, response, path }) => {
            if (requestStart) {
                /** 计算请求处理时间 */
                const [seconds, nanoseconds] = process.hrtime(requestStart);
                const duration = seconds * 1000 + nanoseconds / 1000000;

                /** 记录响应日志 */
                log.http({
                    type: 'response',
                    method: request.method,
                    path,
                    color: color,
                    statusCode: set.status || 200,
                    requestTime: `${duration.toFixed(2)}ms`,
                    responseSize: getResponseSize(response),
                });
            }
        });

function getResponseSize(response: Response) {
    let responseSize = 0;

    if (response instanceof Response) {
        // 对于 Response 对象，可以通过 headers 获取 content-length
        const contentLength = response.headers.get('content-length');
        if (contentLength) {
            responseSize = parseInt(contentLength, 10);
        } else if (response.body) {
            // 如果没有 content-length，可以尝试读取 body 大小
            // 注意：这可能会消耗 stream，需要谨慎使用
            responseSize = new Blob([response.body]).size;
        }
    } else if (typeof response === 'string') {
        // 对于字符串响应，计算字节大小
        responseSize = new TextEncoder().encode(response).length;
    } else if (response && typeof response === 'object') {
        // 对于对象响应，先序列化再计算大小
        responseSize = new TextEncoder().encode(JSON.stringify(response)).length;
    } else if (response instanceof File || response instanceof Blob) {
        // 对于文件响应，可以直接访问 size 属性
        responseSize = response.size;
    }

    return formatFileSize(responseSize);
}
export default loggerPlugin;
