/**
 * @file        健康检查类型定义
 * @author      hotok
 * @date        2025-06-28
 * @lastEditor  hotok
 * @lastEditTime 2025-06-28
 * @description 健康检查相关的TypeScript类型定义
 */

/// <reference types="node" />

/**
 * 组件健康状态
 */
export type ComponentHealthStatus = 'healthy' | 'unhealthy' | 'degraded';

/**
 * 系统整体健康状态
 */
export type SystemHealthStatus = 'healthy' | 'unhealthy' | 'degraded';

/**
 * 组件状态信息
 */
export interface ComponentStatus {
    /** 组件状态 */
    status: ComponentHealthStatus;
    /** 响应时间（毫秒） */
    responseTime?: number;
    /** 错误信息 */
    error?: string;
    /** 详细信息 */
    details?: Record<string, any>;
}

/**
 * 系统信息
 */
export interface SystemInfo {
    /** 操作系统平台 */
    platform: string;
    /** 系统架构 */
    arch: string;
    /** Node.js版本 */
    nodeVersion: string;
    /** 运行时 */
    runtime: string;
    /** 进程ID */
    pid: number;
    /** 当前工作目录 */
    cwd: string;
}

/**
 * 性能指标
 */
export interface PerformanceMetrics {
    /** CPU使用情况 */
    cpuUsage: {
        user: number;
        system: number;
    };
    /** 内存使用情况 */
    memoryUsage: {
        rss: number;
        heapTotal: number;
        heapUsed: number;
        external: number;
        arrayBuffers: number;
    };
    /** 运行时间（秒） */
    uptime: number;
}

/**
 * 基本健康状态响应
 */
export interface HealthStatus {
    /** 响应码 */
    code: number;
    /** 响应消息 */
    message: string;
    /** 健康状态数据 */
    data: {
        /** 系统整体状态 */
        status: SystemHealthStatus;
        /** 时间戳 */
        timestamp: string;
        /** 系统运行时间（秒） */
        uptime: number;
        /** 响应时间（毫秒） */
        responseTime: number;
        /** 版本号 */
        version: string;
        /** 环境 */
        environment: string;
        /** 错误信息（仅在异常时） */
        error?: string;
        /** 各组件状态 */
        components: {
            /** MySQL数据库状态 */
            mysql?: ComponentStatus;
            /** Redis缓存状态 */
            redis?: ComponentStatus;
            /** 其他组件状态 */
            [key: string]: ComponentStatus | undefined;
        };
    };
}

/**
 * 详细健康状态响应
 */
export interface DetailedHealthStatus extends HealthStatus {
    /** 详细健康状态数据 */
    data: HealthStatus['data'] & {
        /** 系统信息 */
        system?: SystemInfo;
        /** 性能指标 */
        performance?: PerformanceMetrics;
    };
} 