/**
 * @file        Winston日志器工具类
 * @author      hotok
 * @date        2025-06-28
 * @lastEditor  hotok
 * @lastEditTime 2025-06-28
 * @description 基于winston的高性能日志记录器，支持分环境输出、按日期轮转、彩色美化
 */

import winston from 'winston';
import DailyRotateFile from 'winston-daily-rotate-file';
import { loggerConfig } from '@/config/logger.config';
import chalk from 'chalk';
import { centerText } from '@/utils/text';

/**
 * 日志等级颜色格式化方法
 */
const colorMethods = {
    error: (msg: string) => chalk.bgRed.white(msg),
    warn: (msg: string) => chalk.bgYellow.black(msg),
    info: (msg: string) => chalk.bgGreen(msg),
    http: (msg: string) => chalk.bgCyan(msg),
    verbose: (msg: string) => chalk.bgGray(msg),
    debug: (msg: string) => chalk.bgMagenta(msg),
    silly: (msg: string) => chalk.bgGray(msg),
};
const colorMethodsForStart = {
    error: (msg: string) => chalk.red(msg),
    warn: (msg: string) => chalk.yellow(msg),
    info: (msg: string) => chalk.green(msg),
    http: (msg: string) => chalk.cyan(msg),
    verbose: (msg: string) => chalk.gray(msg),
    debug: (msg: string) => chalk.magenta(msg),
    silly: (msg: string) => chalk.gray(msg),
};

/**
 * 格式化堆栈信息，每行第一个字符用红色背景
 * @param stack 堆栈字符串
 * @returns 格式化后的堆栈字符串
 */
const formatStack = (stack: string): string => {
    return (
        chalk.red('•••') +
        '\n' +
        stack
            .split('\n')
            .map((line, index) => {
                if (index === 0) return line; // 第一行是错误消息，不处理
                if (line.trim() === '') return line; // 空行不处理

                // 为每行第一个字符添加红色背景
                const firstChar = line.charAt(0);
                const restOfLine = line.slice(1);
                return chalk.bgRed(' ') + firstChar + restOfLine;
            })
            .join('\n')
    );
};

/**
 * 格式化JSON信息，每行第一个字符用对应日志颜色
 * @param str JSON字符串
 * @param level 日志等级
 * @returns 格式化后的JSON字符串
 */
const formatJSON = (str: string, level: string): string => {
    if (typeof str !== 'string') {
        console.log('str', str);
        return JSON.stringify(str, null, 2);
    }
    if (!str?.includes('\n')) {
        return str;
    }
    const color = colorMethodsForStart[level as keyof typeof colorMethods];
    return (
        '\n' +
        color('|') +
        str
            .split('\n')
            .map((line, index) => {
                if (index === 0) return line; // 第一行是错误消息，不处理
                if (line.trim() === '') return line; // 空行不处理

                // 为每行第一个字符添加红色背景
                const firstChar = line.charAt(0);
                const restOfLine = line.slice(1);
                return color('|') + firstChar + restOfLine;
            })
            .join('\n')
    );
};

/**
 * 格式化JSON信息，每行第一个字符用对应日志颜色
 * @param str JSON字符串
 * @param level 日志等级
 * @returns 格式化后的JSON字符串
 */
const formatHTTP = (obj: any): string => {
    if (obj.type === 'request') {
        return obj.color + `|<    ${obj.method} ${obj.path} ${obj.ip} ${obj.browser}`;
    } else if (obj.type === 'response') {
        return (
            obj.color +
            `|   > ${obj.method} ${obj.path} ${obj.statusCode} ${chalk.bold(obj.requestTime)} ${chalk.bgGreen(obj.responseSize)}`
        );
    }
};
/**
 * 控制台日志传输器
 */

const consoleTransport = new winston.transports.Console({
    format: winston.format.combine(
        winston.format.timestamp({ format: 'YY/MM/DD HH:mm:ss' }),
        winston.format.printf(({ timestamp, message, level, stack }) => {
            // 使用居中对齐格式化日志级别
            const levelText = centerText(level.toUpperCase(), 7);
            const levelFormatted = colorMethods[level as keyof typeof colorMethods](levelText);

            if (level === 'error' && stack && typeof stack === 'string') {
                const formattedStack = formatStack(stack);
                return `[${chalk.gray(timestamp)}] ${levelFormatted} ${message}\n${formattedStack}`;
            } else if (level === 'error') {
                return `[${chalk.gray(timestamp)}] ${levelFormatted} 未定义的异常 ${message}`;
            } else if (level === 'http') {
                return `[${chalk.gray(timestamp)}] ${levelFormatted} ${formatHTTP(message)}`;
            }

            return `[${chalk.gray(timestamp)}] ${levelFormatted} ${formatJSON(message as string, level)}`;
        }),
    ),
});

/**
 * 应用主日志文件传输器
 */
const appFileTransport = new DailyRotateFile({
    filename: `${loggerConfig.directory}/app-%DATE%.log`,
    datePattern: 'YYYY-MM-DD',
    maxSize: loggerConfig.maxSize,
    maxFiles: loggerConfig.maxFiles,
    format: winston.format.combine(winston.format.timestamp({ format: 'YYYY-MM-DD HH:mm:ss' }), winston.format.json()),
});

/**
 * 错误专用日志文件传输器
 */
const errorFileTransport = new DailyRotateFile({
    filename: `${loggerConfig.directory}/error-%DATE%.log`,
    datePattern: 'YYYY-MM-DD',
    maxSize: loggerConfig.maxSize,
    maxFiles: loggerConfig.maxFiles,
    level: 'error',
    format: winston.format.combine(
        winston.format.timestamp({ format: 'YYYY-MM-DD HH:mm:ss' }),
        winston.format.errors({ stack: true }), // 确保堆栈信息被记录
        winston.format.json(),
    ),
});

/**
 * Winston日志器实例
 */
const logger = winston.createLogger({
    /** 日志级别 */
    level: loggerConfig.level,

    /** 传输器配置 */
    transports: [
        // 应用主日志文件
        appFileTransport,

        // 错误专用日志文件
        errorFileTransport,

        // 控制台日志（如果启用）
        ...(loggerConfig.console ? [consoleTransport] : []),
    ],
});

/**
 * 格式化日志消息，支持字符串和对象
 * @param message 日志消息，可以是字符串或对象
 * @returns 格式化后的字符串
 */
const formatMessage = (message: string | object): string => {
    if (typeof message === 'string') {
        return message;
    }
    return JSON.stringify(message, null, 2);
};

/**
 * 简单日志器类
 */
export class Logger {
    /**
     * 调试日志
     * @param message 日志消息，支持字符串或对象
     */
    static debug(message: string | object): void {
        logger.debug(formatMessage(message));
    }

    /**
     * 信息日志
     * @param message 日志消息，支持字符串或对象
     */
    static info(message: string | object): void {
        logger.info(formatMessage(message));
    }

    /**
     * 警告日志
     * @param message 日志消息，支持字符串或对象
     */
    static warn(message: string | object): void {
        logger.warn(formatMessage(message));
    }

    /**
     * 错误日志 - 只接受 Error 对象，自动记录堆栈信息
     * @param error Error 对象，包含错误信息和堆栈
     */
    static error(error: Error): void {
        logger.error({
            message: error.message,
            stack: error.stack,
            name: error.name,
            cause: error.cause,
        });
    }

    /**
     * HTTP日志
     * @param message 日志消息，支持字符串或对象
     */
    static http(message: string | object): void {
        logger.http(message);
    }

    /**
     * 详细日志
     * @param message 日志消息，支持字符串或对象
     */
    static verbose(message: string | object): void {
        logger.verbose(formatMessage(message));
    }
}

// 导出默认实例
export default Logger;
