/**
 * @file        健康检查响应验证器
 * @author      hotok
 * @date        2025-06-28
 * @lastEditor  hotok
 * @lastEditTime 2025-06-28
 * @description 健康检查接口响应数据结构验证
 */

import { t } from 'elysia';

/**
 * 组件状态验证器
 */
const componentStatusSchema = t.Object({
    /** 组件状态 */
    status: t.Union([t.Literal('healthy'), t.Literal('unhealthy'), t.Literal('degraded')]),
    /** 响应时间（毫秒） */
    responseTime: t.Optional(t.Number()),
    /** 错误信息 */
    error: t.Optional(t.String()),
    /** 详细信息 */
    details: t.Optional(t.Record(t.String(), t.Any())),
});

/**
 * 系统信息验证器
 */
const systemInfoSchema = t.Object({
    /** 操作系统平台 */
    platform: t.String(),
    /** 系统架构 */
    arch: t.String(),
    /** Node.js版本 */
    nodeVersion: t.String(),
    /** 运行时 */
    runtime: t.String(),
    /** 进程ID */
    pid: t.Number(),
    /** 当前工作目录 */
    cwd: t.String(),
});

/**
 * 性能指标验证器
 */
const performanceMetricsSchema = t.Object({
    /** CPU使用情况 */
    cpuUsage: t.Object({
        user: t.Number(),
        system: t.Number(),
    }),
    /** 内存使用情况 */
    memoryUsage: t.Object({
        rss: t.Number(),
        heapTotal: t.Number(),
        heapUsed: t.Number(),
        external: t.Number(),
        arrayBuffers: t.Number(),
    }),
    /** 运行时间（秒） */
    uptime: t.Number(),
});

/**
 * 基本健康状态响应验证器
 */
const basicHealthDataSchema = t.Object({
    /** 系统整体状态 */
    status: t.Union([t.Literal('healthy'), t.Literal('unhealthy'), t.Literal('degraded')]),
    /** 时间戳 */
    timestamp: t.String(),
    /** 系统运行时间（秒） */
    uptime: t.Number(),
    /** 响应时间（毫秒） */
    responseTime: t.Number(),
    /** 版本号 */
    version: t.String(),
    /** 环境 */
    environment: t.String(),
    /** 错误信息（仅在异常时） */
    error: t.Optional(t.String()),
    /** 各组件状态 */
    components: t.Object({
        /** MySQL数据库状态 */
        mysql: t.Optional(componentStatusSchema),
        /** Redis缓存状态 */
        redis: t.Optional(componentStatusSchema),
    }),
});

/**
 * 详细健康状态数据验证器
 */
const detailedHealthDataSchema = t.Intersect([
    basicHealthDataSchema,
    t.Object({
        /** 系统信息 */
        system: t.Optional(systemInfoSchema),
        /** 性能指标 */
        performance: t.Optional(performanceMetricsSchema),
    }),
]);

/**
 * 健康检查响应验证器
 */
export const healthResponse = {
    200: t.Object({
        /** 响应码 */
        code: t.Number(),
        /** 响应消息 */
        message: t.String(),
        /** 健康状态数据 */
        data: basicHealthDataSchema,
    }),
    500: t.Object({
        /** 响应码 */
        code: t.Number(),
        /** 响应消息 */
        message: t.String(),
        /** 错误数据 */
        data: t.Object({
            /** 系统整体状态 */
            status: t.Literal('unhealthy'),
            /** 时间戳 */
            timestamp: t.String(),
            /** 系统运行时间（秒） */
            uptime: t.Number(),
            /** 响应时间（毫秒） */
            responseTime: t.Number(),
            /** 版本号 */
            version: t.String(),
            /** 环境 */
            environment: t.String(),
            /** 错误信息 */
            error: t.String(),
            /** 各组件状态 */
            components: t.Object({}),
        }),
    }),
};

/**
 * 详细健康检查响应验证器
 */
export const detailedHealthResponse = {
    200: t.Object({
        /** 响应码 */
        code: t.Number(),
        /** 响应消息 */
        message: t.String(),
        /** 详细健康状态数据 */
        data: detailedHealthDataSchema,
    }),
    500: t.Object({
        /** 响应码 */
        code: t.Number(),
        /** 响应消息 */
        message: t.String(),
        /** 错误数据 */
        data: t.Object({
            /** 系统整体状态 */
            status: t.Literal('unhealthy'),
            /** 时间戳 */
            timestamp: t.String(),
            /** 系统运行时间（秒） */
            uptime: t.Number(),
            /** 响应时间（毫秒） */
            responseTime: t.Number(),
            /** 版本号 */
            version: t.String(),
            /** 环境 */
            environment: t.String(),
            /** 错误信息 */
            error: t.String(),
            /** 各组件状态 */
            components: t.Object({}),
        }),
    }),
}; 